% MATLAB code: Chapter 11 (Miscellanea)
% File: rTsay.m
% Coded by: Wai-Sum Chan, et al. (2015)
% A robust VSETAR-type of test statistic 
%
% INPUT:
% Y     = data-matrix of size: n*k, with n = length(data)
%       = EX1Y.dat (Example 1 in paper) and EX2Y.dat (Example 2 in paper)
% k     = number of time series variables
%         k = 4 (Example 1), k = 2 (Example 2)
% Z     = threshold variable 
%         Z = Y(:,4) (Example 1), and Z = Y(:,2) (Example 2)
% delta = outlier tolerance parameter
% gamma = trimming poportion
% pmin  = minimum value of AR order
% pmax  = maximum value of AR order
% 
% OUTPUT:
% OutputTS = p-value, p, d, df, cdv3 (Test statistic)
%
% Reference:
% Chan, W.S., Cheung, S.H., Chow, W.K., and Zhang, L-X. (2015).
%    A robust test for threshold-type nonlinearity in multivariate 
%    time series analysis. 
%    Journal of Forecasting, 34(6), 441-454.
%    DOI: 10.1002/for.2344.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
format('long');

P           = [ ];
gamma       = 0.25; %  switch to determine the value of gamma
deltaswitch = 1;    %  switch to determine the value of delta, 1-> 0.01, 2-> 0.05,
delta       = [.01,.05];
OMEGA0      = 1000; % magnitude of the outlier effect
OutputTS    = [ ];
Output      = [ ];
N0          = 1000; % number of iterations
siglevel    = [0.99,0.95,0.9]; 

Y           = data;
n           = length(data);
k           = 2;       % CHANGE
Z           = Y(:,2);  % CHANGE


% Switch the value of delta
if deltaswitch==1         
  options.ns    = 500;
  options.nc    = 10;
  options.delta = 0.01;
elseif deltaswitch==2  
  options.ns    = 500;
  options.nc    = 10;
  options.delta = 0.05;
end;
% Switch the value of delta

% Perform test for non-linearity given p d n and m
n    = length(Y);
m0   = floor(3*sqrt(n));      
pmin = 1;
pmax = 4;
P    = [ ];
CD   = [ ];
dd   = [ ];
DF   = [ ];

for p = pmin:pmax
   for d = 1:p
      % Initialize variables
      nca       = p*k+1;
      maxpk     = 30;
      maxp      = 10;
      zz        = zeros(n,1);
      zzs       = zeros(n,1);
      ipi       = zeros(n,1);
      xmtx      = zeros(n,nca);
      ymtx      = zeros(n,k);
      ehat      = zeros(n,k);
      ehatOLS   = zeros(n,k);
      etahat    = zeros(n,k);
      tempp1    = zeros(n,k);
      tempp2    = zeros(n,k);
      etahatv3  = zeros(n,k);
      etahatOLS = zeros(n,k);
      etasum    = zeros(k,k);
      y2mtx     = zeros(n,k);
    
      % Arranged Regression Setup, Eqn. (4) of the paper
      h    = max(d,p);
      neff = n-h;
      for i = 1:neff
         ipi(i) = i;
         zz(i)  = Z(h-d+i);
      end
    
      for i = 1:neff
         zzs(i) = zz(i);
      end;
      zzs = zzs(1:neff);
      ipi = ipi(1:neff);
      zzt = [zzs,ipi];
      zzt = sortrows(zzt,1);
      zzs = zzt(:,1);
      ipi = zzt(:,2)+h-d;
        
      for i = 1:neff
         for j = 1:k
            ymtx(i,j) = Y((ipi(i)+d),j);
         end
      end
      ymtx = ymtx(1:neff,:);
      xmtx(:,1)=1;  % Construct the intercept coefficients   
      for i = 1:neff
        kk = 2;    % if intercept is included kk=2
        for l = 1:p
           for j = 1:k
              xmtx(i,kk) = Y((ipi(i)+d-l),j);
              kk         = kk+1;
            end
        end
      end
    
    xmtx   = xmtx(1:neff,1:nca);
    index3 = [ ];
    
    % Start the Rolling Arranged Regression with m0 startup data
    for m = m0:(neff-1)
        % Least Trimmed Squares Fitting, giving phihat
        nra = m;
        nca = p*k+1;  % +1 if intercept is included
        lda = n;
        tol = 0;
        A   = xmtx(1:nra,1:nca);
        B   = ymtx(1:m,:);
        outtemp1 = mlts(A,B,gamma,options); % Do the RMLTS regression for A and B
        bvec     = outtemp1.betaR;          % Store the vector of beta 
        res      = B-A*bvec;                % Store the residuals for the regression
       
        % Calculate the predictive residuals in Eqn. (5) of the paper
        % Produce the 1-step ahead of vector X
        Y1step = Y((ipi(m+1)+d),:);
        X1step = 1;
        for i = 1:p
           X1step = [X1step,Y((ipi(m+1)+d-i),:)];
        end
        % End of Production of the 1-step ahead of vector X
        % Compute the corresponding 1-step ahead residuals       
        ehat((ipi(m+1)+d),:) = Y1step-X1step*bvec; 

        % Standardization of the predictive residuals eqn. (6) of the paper       
        % LTS
        XXv3   = A(outtemp1.nooutlier,:); % Trim out the observations according to RMLTS
        vmv3   = inv(XXv3'*XXv3); 
        % Compute the prediction standard error of the 1-step ahead residuals 
        varev3 = sqrt(1+X1step*vmv3*X1step'); 
        
        % Compute the standardized 1-step ahead residuals
        etahatv3((ipi(m+1)+d),:) = ehat((ipi(m+1)+d),:)/varev3; 
        index3(m-m0+1,:) = (ipi(m+1)+d);
    end
    % End of Rolling Regression
    
    xmtxtemp = xmtx((m0+1):neff,:);
    ymtxtemp = ymtx((m0+1):neff,:);
    
    % Identify the outlier according to the full data set using RMLTS
    outtemp  = mlts_2(xmtxtemp,ymtxtemp,gamma,options); 
    
    % Least squares estimation for eqn. (7) 
    % LTS
    x3mtx = [ ];
    y3mtx = zeros((neff-m0),k);
    for m = (m0+1):neff
        y3mtx((m-m0),:)  = etahatv3((ipi(m)+d),:);
        x3temp           = 1;    % intercept
       
        for j = 1:p
            x3temp = [x3temp,Y((ipi(m)+d-j),:)]; 
        end
        x3mtx      = [x3mtx;x3temp];
    end
       
    index4 = outtemp.nooutlier;
    % G is the corresponding X for the standardised 1-step ahead residuals
    G      = x3mtx(index4,:); 
    % H stores the standardized 1-step ahead residuals from the rolling regression
    H      = y3mtx(index4,:);   
    b3vec  = G\H;          % Store the vector of beta 
    ressv3 = H-G*b3vec;    % Store the vector of residuals
        
    
    % Compute the S1 matrix and its determinant 
    % LTS
    aaaav3  = sum(outtemp.nooutlier); 
    % Calculate # of terms that are not trimmed out according to the RMLTS in eqn. (7)
    ppv3    = size(G,2);  % Calculate # of term in the X vector (parameters)
    
    DDv3    = ressv3; % Store the vector of residuals in eqn. (7) that are not trimmed out
    % Compute S1
    S1v3    = (DDv3'*DDv3)/(aaaav3-ppv3);      
    S1v3    = outtemp.cdelta*S1v3;
    detS1v3 = det(S1v3);      % Compute det(S1)

    % Compute the S0 matrix and its determinant
    % LTS
    
    % Store the vector of 1-step ahead standardized residuals in rolling 
    % regression not being trimmed out
    eeev3   = H; 
    S0v3    = eeev3'*eeev3/(aaaav3-ppv3);
    S0v3    = outtemp.cdelta*S0v3;
    detS0v3 = det(S0v3);
    sigmagammadelta = ((1-1/outtemp.cdelta)^2)*(outtemp.cgamma/(1-gamma))...
                      +(2-1/outtemp.cdelta)/(1-delta(deltaswitch));
    const3    = (neff-m0)/(sigmagammadelta*outtemp.cdelta)-(nca);

    cdv3      = const3*(log(detS0v3)-log(detS1v3));  % Compute the test statistic       
    df        = k*(nca);
    dd        = [dd;d];
    criticalv = chi2inv(siglevel,df);
    
    CD        = [CD;cdv3];
    OutputTS  = [OutputTS;[1-chi2cdf(cdv3,df),p,d,df,cdv3]];
    
    end
end

disp(OutputTS);


